package com.xkrs.straw.service.impl;

import com.xkrs.common.encapsulation.PromptMessageEnum;
import com.xkrs.dao.StreetDao;
import com.xkrs.model.entity.StreetEntity;
import com.xkrs.model.qo.SysUserQo;
import com.xkrs.service.RedisService;
import com.xkrs.straw.dao.RelRoleAuthorityDao;
import com.xkrs.straw.dao.RelUserRoleDao;
import com.xkrs.straw.dao.SysRoleDao;
import com.xkrs.straw.dao.SysUserDao;
import com.xkrs.straw.model.entity.RelRoleAuthorityEntity;
import com.xkrs.straw.model.entity.RelUserRoleEntity;
import com.xkrs.straw.model.entity.SysRoleEntity;
import com.xkrs.straw.model.entity.SysUserEntity;
import com.xkrs.straw.service.SysUserService;
import com.xkrs.straw.utils.VipTimeRangeUtils;
import org.apache.http.util.TextUtils;
import org.springframework.context.i18n.LocaleContextHolder;
import org.springframework.data.domain.Sort;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.security.crypto.keygen.KeyGenerators;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import javax.persistence.criteria.Predicate;
import javax.servlet.http.HttpServletRequest;
import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.List;
import java.util.Locale;

import static com.xkrs.common.encapsulation.OutputEncapsulation.outputEncapsulationObject;
import static com.xkrs.utils.DateTimeUtil.dateTimeToString;
import static com.xkrs.utils.EncryptDecryptUtil.encry256;

/**
 * 系统用户服务实现
 */
@Service
public class SysUserServiceImpl implements SysUserService {

    @Resource
    private SysUserDao sysUserDao;

    @Resource
    private SysRoleDao sysRoleDao;

    @Resource
    private RelUserRoleDao relUserRoleDao;

    @Resource
    private RelRoleAuthorityDao relRoleAuthorityDao;

    @Resource
    private RedisService redisService;

    @Resource
    private StreetDao streetDao;

    private final Locale locale = LocaleContextHolder.getLocale();

    /**
     * 检查用户名是否存在
     */
    @Override
    public boolean checkUserName(String userName) {
        int num = sysUserDao.checkUserName(userName);
        return (num == 0);
    }

    /**
     * 新增用户
     */
    @Transactional(rollbackFor = Exception.class)
    @Override
    public String addUser(SysUserQo sysUserQo, HttpServletRequest servletRequest) {
        // 获取区域信息
        String salt = KeyGenerators.string().generateKey();
        SysUserEntity sysUserEntity = new SysUserEntity();
        sysUserEntity.setUserName(sysUserQo.getUserName());
        sysUserEntity.setUserAgent(servletRequest.getHeader("user-agent"));
        String o = (String) redisService.get(sysUserQo.getUserName());
        if ("".equals(o) || o == null) {
            return outputEncapsulationObject(PromptMessageEnum.DATA_WRONG, "请先发送验证码！", locale);
        }
        if (!redisService.get(sysUserQo.getUserName()).equals(sysUserQo.getVerificationCode())) {
            return outputEncapsulationObject(PromptMessageEnum.DATA_WRONG, "验证码错误，请重新输入！", locale);
        }
        if ("".equals(sysUserQo.getReallyName()) || sysUserQo.getReallyName() == null) {
            return outputEncapsulationObject(PromptMessageEnum.DATA_NONE, "真实姓名不能为空！", locale);
        }
        sysUserEntity.setReallyName(sysUserQo.getReallyName());
        sysUserEntity.setSalt(salt);
        sysUserEntity.setPassword(encry256(sysUserQo.getPassword() + salt));
        sysUserEntity.setTelephone(sysUserQo.getUserName());

        String countyCode = sysUserQo.getCountyCode();
        String countyName = sysUserQo.getCountyName();
        if (TextUtils.isEmpty(countyCode)) {
            return outputEncapsulationObject(PromptMessageEnum.DATA_WRONG, "省市区街道编码不能为空！", locale);
        }
        if (TextUtils.isEmpty(countyName)) {
            return outputEncapsulationObject(PromptMessageEnum.DATA_WRONG, "省市区街道名称不能为空！", locale);
        }
        if (9 == countyCode.length()) {
            List<StreetEntity> streetCodeList = streetDao.findByStreetCode(countyCode);
            if (streetCodeList == null || streetCodeList.isEmpty()) {
                return outputEncapsulationObject(PromptMessageEnum.DATA_WRONG, "街道编码不存在！", locale);
            }
            List<StreetEntity> streetNameList = streetDao.findByStreetName(countyName);
            if (streetNameList == null || streetNameList.isEmpty()) {
                return outputEncapsulationObject(PromptMessageEnum.DATA_WRONG, "您所选择的区域不存在！", locale);
            }
            sysUserEntity.setActiveFlag(0);
            sysUserEntity.setAccountType("街道级");
        } else if ("0000".equals(countyCode.substring(2))) {
            List<StreetEntity> proCodeList = streetDao.findByProCode(countyCode);
            if (proCodeList == null || proCodeList.size() == 0) {
                return outputEncapsulationObject(PromptMessageEnum.DATA_WRONG, "省编码不存在！", locale);
            }
            List<StreetEntity> proNameList = streetDao.findByProName(countyName);
            if (proNameList == null || proNameList.size() == 0) {
                return outputEncapsulationObject(PromptMessageEnum.DATA_WRONG, "您所选择的区域不存在！", locale);
            }
            sysUserEntity.setActiveFlag(1);
            sysUserEntity.setAccountType("省级");
        } else if ("00".equals(countyCode.substring(4)) && !"0000".equals(countyCode.substring(2))) {
            List<StreetEntity> cityCodeList = streetDao.findByCityCode(countyCode);
            if (cityCodeList == null || cityCodeList.size() == 0) {
                return outputEncapsulationObject(PromptMessageEnum.DATA_WRONG, "市编码不存在！", locale);
            }
            List<StreetEntity> cityNameList = streetDao.findByCityName(countyName);
            if (cityNameList == null || cityNameList.size() == 0) {
                return outputEncapsulationObject(PromptMessageEnum.DATA_WRONG, "您所选择的区域不存在！", locale);
            }
            sysUserEntity.setActiveFlag(1);
            sysUserEntity.setAccountType("市级");
        } else {
            List<StreetEntity> countyCodeList = streetDao.findByCountyCode(countyCode);
            if (countyCodeList == null || countyCodeList.isEmpty()) {
                return outputEncapsulationObject(PromptMessageEnum.DATA_WRONG, "区县编码不存在！", locale);
            }
            List<StreetEntity> countyNameList = streetDao.findByCountyName(countyName);
            if (countyNameList == null || countyNameList.isEmpty()) {
                return outputEncapsulationObject(PromptMessageEnum.DATA_WRONG, "您所选择的区域不存在！", locale);
            }
            sysUserEntity.setActiveFlag(0);
            sysUserEntity.setAccountType("县级");
        }
        if (9 == countyCode.length()) {//街道账号7天后过期
            //生成新的VIP时间范围Json内容
            String newVipTimeRangeJson = VipTimeRangeUtils.obtainNewVipTimeRangeJson(null, LocalDateTime.now().plusDays(7L));
            //更新字段
            sysUserEntity.setVipTimeRangeJson(newVipTimeRangeJson);
        } else {//省市区县账号5天后过期
            //生成新的VIP时间范围Json内容
            String newVipTimeRangeJson = VipTimeRangeUtils.obtainNewVipTimeRangeJson(null, LocalDateTime.now().plusDays(5L));
            //更新字段
            sysUserEntity.setVipTimeRangeJson(newVipTimeRangeJson);
        }
        sysUserEntity.setAddTime(dateTimeToString(LocalDateTime.now()));
        sysUserEntity.setLoginNum(0);
        sysUserEntity.setCountyCode(countyCode);
        sysUserEntity.setCountyName(countyName);
        sysUserEntity.setVipLevel(0);
        sysUserEntity.setReceiveSms(0);
        sysUserEntity.setRemark("");//备注
        sysUserDao.save(sysUserEntity);

        RelUserRoleEntity relUserRoleEntity = new RelUserRoleEntity();
        relUserRoleEntity.setUserId(sysUserEntity.getId());
        relUserRoleEntity.setRoleId(2L);

        RelRoleAuthorityEntity relRoleAuthorityEntity = new RelRoleAuthorityEntity();
        relRoleAuthorityEntity.setRoleId(2L);
        relRoleAuthorityEntity.setAuthorityId(2L);
        relRoleAuthorityEntity.setUserId(sysUserEntity.getId());
        relUserRoleDao.save(relUserRoleEntity);

        relRoleAuthorityDao.save(relRoleAuthorityEntity);
        return outputEncapsulationObject(PromptMessageEnum.SUCCESS, "注册成功！", locale);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public String deleteSysUser(String userName) {
        int deleteCount = sysUserDao.deleteSysUser(userName);
        return outputEncapsulationObject(PromptMessageEnum.SUCCESS, "删除" + deleteCount + "个账户", locale);
    }

    /**
     * 查询所有用户Vo
     */
    @Transactional(rollbackFor = Exception.class)
    @Override
    public SysUserEntity getSysUserByUserName(String userName) {
        return sysUserDao.selectByUserName(userName);
    }

    /**
     * 更新用户
     */
    @Transactional(rollbackFor = Exception.class)
    @Override
    public int updateSysUser(SysUserQo sysUserQo) {
        SysUserEntity sysUserEntity = new SysUserEntity();
        sysUserEntity.setReallyName(sysUserQo.getReallyName());
        sysUserDao.save(sysUserEntity);
        return 1;
    }

    /**
     * 软删除普通用户
     */
    @Transactional(rollbackFor = Exception.class)
    @Override
    public int softDeleteGeneralUser(Long id) {
        String adminRole = "role_administor";
        String sysRole = "role_system_manager";
        List<SysRoleEntity> list = sysRoleDao.selectByUserId(id);
        if (list.size() > 0) {
            SysRoleEntity sysRoleEntity = list.get(1);
            if (sysRole.equals(sysRoleEntity.getRoleName()) || adminRole.equals(sysRoleEntity.getRoleName())) {
                sysUserDao.softDeleteGeneralUserById(id);
                return 0;
            }
        }
        return 1;
    }

    /**
     * 查询用户的信息
     */
    @Override
    public List<SysUserEntity> selectAllUser() {
        return sysUserDao.findAll(Sort.by(Sort.Direction.DESC, "id"));
    }

    /**
     * 启用
     */
    @Transactional(rollbackFor = Exception.class)
    @Override
    public void updateEnable(Long userId) {
        sysUserDao.updateEnable(userId);
    }

    /**
     * 禁用
     */
    @Transactional(rollbackFor = Exception.class)
    @Override
    public void updateDisable(Long userId) {
        sysUserDao.updateDisable(userId);
    }

    /**
     * 用户修改密码
     */
    @Transactional(rollbackFor = Exception.class)
    @Override
    public String updatePassword(String oldPassword, String newPassword, String confirmPassword, SysUserEntity sysUserEntity) {
        String oldFrontPassword = encry256(oldPassword + sysUserEntity.getSalt());
        if (!oldFrontPassword.equals(sysUserEntity.getPassword())) {
            return outputEncapsulationObject(PromptMessageEnum.PROCESS_FAIL, "旧密码输入错误", locale);
        }
        if (!newPassword.equals(confirmPassword)) {
            return outputEncapsulationObject(PromptMessageEnum.PROCESS_FAIL, "两次密码输入不一致", locale);
        }
        String newAfterPassword = encry256(newPassword + sysUserEntity.getSalt());
        sysUserDao.updatePassword(sysUserEntity.getId(), newAfterPassword);
        return outputEncapsulationObject(PromptMessageEnum.SUCCESS, "密码修改成功", locale);
    }

    /**
     * 管理员修改用户的密码
     */
    @Transactional(rollbackFor = Exception.class)
    @Override
    public String adminUpdatePassword(Long userId, String newPassword, String confirmPassword) {
        if (!newPassword.equals(confirmPassword)) {
            return outputEncapsulationObject(PromptMessageEnum.PROCESS_FAIL, "两次密码输入不一致", locale);
        }
        SysUserEntity sysUserEntity = sysUserDao.selectByUserId(userId);
        String newAfterPassword = encry256(newPassword + sysUserEntity.getSalt());
        sysUserDao.updatePassword(userId, newAfterPassword);
        return outputEncapsulationObject(PromptMessageEnum.SUCCESS, "密码修改成功", locale);
    }

    @Override
    public SysUserEntity getUserByUserName(String userName) {
        Specification<SysUserEntity> specification = (root, criteriaQuery, criteriaBuilder) -> {
            //查询条件集合
            List<Predicate> predicateList = new ArrayList<>();
            predicateList.add(criteriaBuilder.equal(root.get("userName").as(String.class), userName));
            Predicate[] predicateArray = new Predicate[predicateList.size()];
            return criteriaBuilder.and(predicateList.toArray(predicateArray));
        };
        List<SysUserEntity> sysUserEntityList = sysUserDao.findAll(specification, Sort.by(Sort.Direction.DESC, "id"));
        return sysUserEntityList.get(0);
    }

    /**
     * 修改账号到期时间
     */
    @Transactional(rollbackFor = Exception.class)
    @Override
    public String updateOverTime(Long userId, String overTime) {
        sysUserDao.updateOverTime(userId, overTime);
        return outputEncapsulationObject(PromptMessageEnum.SUCCESS, "修改成功", locale);
    }

    /**
     * 记录用户登录次数
     */
    @Transactional(rollbackFor = Exception.class)
    @Override
    public void updateLoginNum(Long userId, Integer loginNum) {
        sysUserDao.updateLoginNum(userId, loginNum);
    }

    /**
     * 记录用户登录时间
     */
    @Transactional(rollbackFor = Exception.class)
    @Override
    public void updateLoginLastTime(Long userId, String loginLastTime) {
        sysUserDao.updateLoginLastTime(userId, loginLastTime);
    }

}
